#!/usr/bin/env python
#-*- coding: utf-8 -*- 

# Allow compatibility between Python 2.7 and 3.x
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
#####

# GUI imports
import Tkinter as tk
import tkFont
import tkFileDialog
import tkMessageBox

# Standard library imports
import os, platform, re, traceback
from threading import Thread
import Queue

# Internal imports
from lib import main

	
if platform.system() == "Darwin":
    
    SYSTEM = "M"
    TITLE_SIZE = 16
    FONT_SIZE = 14
    GEOM_SIZE = "450x700"
    COLOR = "#E6E6E6"
    USER_HOME = os.environ["HOME"]
    TEMP_FOLDER = "/tmp"

elif platform.system() == "Windows":
    
    SYSTEM = "W"
    TITLE_SIZE = 14
    FONT_SIZE = 12
    GEOM_SIZE = "450x700"
    COLOR = "#E6E6E6"
    USER_HOME = os.environ["HOMEPATH"]
    TEMP_FOLDER = os.environ["TEMP"]
    
elif platform.system() == "Linux":
    
    SYSTEM = "L"
    TITLE_SIZE = 14
    FONT_SIZE = 12
    GEOM_SIZE = "450x700"
    COLOR = "#E6E6E6"
    USER_HOME = os.environ["HOME"]
    TEMP_FOLDER = "/tmp"

	
class App:
    
    def __init__(self, parent):
        
        # Fonts
        helv14 = tkFont.Font(family="Helvetica", size=FONT_SIZE)
        helv16 = tkFont.Font(family="Helvetica", size=TITLE_SIZE, 
                             weight="bold", slant="italic")
		
        # This defines the GUI parent (ish)
        self.appParent = parent
        
        self.queue = Queue.Queue()
		
		    # This creates the header text
        self.header = tk.Label(self.appParent, text="Grammar and Mechanics Analysis Tool", 
                               font=helv16, background=COLOR)
        
        self.header.pack()
        		
        # This creates a frame for the meat of the GUI
        self.appContent = tk.Frame(self.appParent, background=COLOR)
        self.appContent.pack()
        
        self.outerContainer = tk.Frame(self.appContent, background=COLOR)
        self.outerContainer.pack(side=tk.RIGHT, expand=tk.TRUE)

        # INSTRUCTIONS AREA
        
        # Text to be displayed above the widgets
        self.instructions_frame = tk.LabelFrame(self.outerContainer, text= "Instructions", 
                                           background=COLOR)
        self.instructions_frame.pack(expand=tk.TRUE)
    
       	# This creates the list of instructions.
        self.instructions = tk.Label(self.instructions_frame, height="5", width="45", 
                                     justify=tk.LEFT, padx="4", pady="6", 
                                     anchor=tk.W, font=helv14, 
                                     text="1. Select desired error indices to check for.\n"\
                                          "2. Choose the input folder that contains .txt files.\n"\
                                          "3. Select the folder that will store the output file.\n"\
                                          "4. Give a name to the output file.\n"\
                                          "5. Press the 'Process Texts' button.")
        self.instructions.pack()
	
        # CHECKBOX AREA
    
        self.checkbox_frame = tk.LabelFrame(self.outerContainer, text="Error Indices", 
                                            background=COLOR, width="45")
        self.checkbox_frame.pack(expand=tk.TRUE)
        
        self.options_frame = tk.LabelFrame(self.outerContainer, text="Output Options", 
                                            background=COLOR)
        self.options_frame.pack(expand=tk.TRUE)
        
		# Checkboxes
        
        self.duplication_var = tk.BooleanVar()
        self.duplication_box = tk.Checkbutton(self.checkbox_frame, text="Duplication", 
                                              variable=self.duplication_var, background=COLOR)
        self.duplication_box.grid(row=1, column=1, sticky="W")	
        self.duplication_box.deselect()
		
        self.grammar_var = tk.BooleanVar()
        self.grammar_box = tk.Checkbutton(self.checkbox_frame, text="Grammar", 
                                          variable=self.grammar_var, background=COLOR)
        self.grammar_box.grid(row=1, column=2, sticky="W")	
        self.grammar_box.deselect()
		
        self.misspelling_var = tk.BooleanVar()
        self.misspelling_box = tk.Checkbutton(self.checkbox_frame, text="Misspelling", 
                                              variable=self.misspelling_var, background=COLOR)
        self.misspelling_box.grid(row=1, column=3, sticky="W")	
        self.misspelling_box.deselect()
			
        self.typographic_var = tk.BooleanVar()
        self.typographic_box = tk.Checkbutton(self.checkbox_frame, text="Typographical", 
                                              variable=self.typographic_var, background=COLOR)
        self.typographic_box.grid(row=2, column=1, sticky="W")	
        self.typographic_box.deselect()
		
        self.whitespace_var = tk.BooleanVar()
        self.whitespace_box = tk.Checkbutton(self.checkbox_frame, text="Whitespace", 
                                             variable=self.whitespace_var, background=COLOR)
        self.whitespace_box.grid(row=2, column=2, sticky="W")
        self.whitespace_box.deselect()
        
        self.detail_var = tk.BooleanVar()
        self.detail_box = tk.Checkbutton(self.options_frame, text="Include detailed errors",
                                          variable=self.detail_var, background=COLOR)
        self.detail_box.grid(row=1,column=1, sticky="W")	
        self.detail_box.deselect()
			
        self.checkbox_all = tk.Button(self.checkbox_frame, text="Select All", 
                                      justify=tk.LEFT, highlightbackground=COLOR)
        self.checkbox_all.grid(row=1, column=0)
        self.checkbox_all.bind("<Button-1>", self.select_all_checkbox)

        self.checkbox_none = tk.Button(self.checkbox_frame, text = "Select None",
                                       justify=tk.LEFT, highlightbackground=COLOR)
        self.checkbox_none.grid(row=2, column=0)
        self.checkbox_none.bind("<Button-1>", self.select_none_checkbox)
        
 
        
        self.var_dict = {
                "duplication": self.duplication_var, 
                "grammar": self.grammar_var, 
                "misspelling": self.misspelling_var, 
                "typographical": self.typographic_var, 
                "whitespace": self.whitespace_var,
                "detail": self.detail_var
                         }
		
        self.box_list = [self.duplication_box, self.grammar_box, self.misspelling_box, 
                         self.typographic_box, self.whitespace_box]
    
        # I/O PATH AREA
    
		# Creates Label Frame for Data Input area
        self.data_frame = tk.LabelFrame(self.outerContainer, text="Data Input", 
                                        background=COLOR)
        self.data_frame.pack(expand=tk.TRUE) 
    
		# This Places the first button under the instructions.
        self.input_dir_button = tk.Button(self.data_frame, highlightbackground=COLOR)
        self.input_dir_button["text"] = "Select Input Folder"
        self.input_dir_button.pack()
		
		# This tells the button what to do when clicked.	 
        # Currently, only a left-click makes the button do anything (e.g. <Button-1>).
        self.input_dir_button.bind("<Button-1>", self.select_input_dir)
		
		# Creates default dirname so if statement in Process Texts can check to see
        # if directory name has been chosen
        self.input_dir_name = ""
		
		# This creates a label for the first program input (Input Directory)
        self.input_dir_label = tk.LabelFrame(self.data_frame, height="1", 
                                             width="45", padx="4", 
                                             text="Your selected input folder:", 
                                             background=COLOR)
        self.input_dir_label.pack()
		
		# Creates label that informs user which directory has been chosen
        input_dir_prompt = "(No Input Folder Chosen)"
        self.input_dir_chosen = tk.Label(self.input_dir_label, height="1",
                                         width="44", justify=tk.LEFT, padx="4", 
                                         anchor=tk.W, font=helv14, text=input_dir_prompt)
        self.input_dir_chosen.pack()
		
		# This creates the Output Directory button.
        self.output_dir_button = tk.Button(self.data_frame, highlightbackground=COLOR)
        self.output_dir_button["text"]= "Select Output Folder"
        self.output_dir_button.pack()
        self.output_dir_button.bind("<Button-1>", self.select_output_dir)
		
        self.output_dir_name = ""
		
		# Creates a label for the second program input (Output Directory)
        self.output_dir_label = tk.LabelFrame(self.data_frame, height="1", 
                                              width="45", padx="4", 
                                              text="Your selected output folder:", 
                                              background=COLOR)
        self.output_dir_label.pack()
		
		# Creates a label that informs sure which directory has been chosen
        output_dir_prompt = "(No Output Folder Chosen)"
        self.output_dir_chosen = tk.Label(self.output_dir_label, height="1", 
                                          width="44", justify=tk.LEFT, padx="4", 
                                          anchor=tk.W, font=helv14, text=output_dir_prompt)
        self.output_dir_chosen.pack()
    
		
        self.innerContainer = tk.Frame(self.data_frame)
        self.innerContainer.pack()
		
		# Creates label for filename entrybox
        self.output_file_label = tk.LabelFrame(self.innerContainer, 
                                               text="Name of output file:", 
                                               background=COLOR)
        self.output_file_label.pack()

		# Creates frame so entry 1 and button can be side-by-side
        self.output_file_frame_top = tk.Frame(self.output_file_label, background=COLOR)
        self.output_file_frame_top.pack()
		
        	# Creates output filename entrybox
        self.output_file_entry = tk.Entry(self.output_file_frame_top, highlightbackground=COLOR)
        self.output_file_entry.insert(tk.INSERT, "error_results")
		
		# Tells output_file_entry_box what to do when user presses "return"
        self.output_file_entry.bind("<Return>", self.enter_output_file)
        self.output_file_entry.pack(side=tk.LEFT)
		
		# Creates Default Value for filename
        self.output_file = ""
		
        self.submit_filename_button = tk.Button(self.output_file_frame_top,
                                                highlightbackground=COLOR)
        self.submit_filename_button["text"] = "Submit Filename"
        self.submit_filename_button.pack(side=tk.RIGHT)
        self.submit_filename_button.bind("<Button-1>", self.submit_filename)
		
        self.output_file_frame = tk.Frame(self.output_file_label)
        self.output_file_frame.pack()
		
        # Label for the filename entry output
        self.output_file_chosen_label = tk.LabelFrame(self.output_file_frame, height="1", 
                                                      width="45", padx="4", 
                                                      text="Your output filename:", background=COLOR)
        self.output_file_chosen_label.pack(side=tk.BOTTOM)
		
        # Creates a label that informs user the current output filename
        output_dir_prompt = "error_results.csv (default)"
        self.output_file_chosen = tk.Label(self.output_file_chosen_label, height="1", width="44", 
                                           justify=tk.LEFT, padx="4", anchor=tk.W, 
                                           font=helv14, text=output_dir_prompt)
        self.output_file_chosen.pack()
		
		
        self.program_space = tk.LabelFrame(self.outerContainer, text="Run Program", 
                                           background=COLOR)
        self.program_space.pack()

        self.program_button = tk.Button(self.program_space, highlightbackground=COLOR)
        self.program_button["text"] = "Process Texts"
        self.program_button.bind("<Button-1>", self.run_program)
        self.program_button.pack()
		
        self.progress_label_frame = tk.LabelFrame(self.program_space, 
                                                  text="Program Status", background=COLOR)
        self.progress_label_frame.pack(expand=tk.TRUE)
		
        progress_text = "Waiting for data to process"
    
        self.progress_status = tk.Label(self.progress_label_frame, height="1", width="45", 
                                        justify=tk.LEFT, padx="4", anchor=tk.W, 
                                        font=helv14, text=progress_text)
        self.progress_status.pack()
                
		
    def select_all_checkbox(self, event):
        
        for checkbox in self.box_list:
            checkbox.select()
                

    def select_none_checkbox(self, event):
        for checkbox in self.box_list:
            checkbox.deselect()
                
			
    def enter_output_file(self, event):
        
        self.output_file = self.output_file_entry.get() + ".csv"
        self.output_file_chosen["text"] = self.output_file
	

    def select_input_dir(self, event):
        
        self.input_dir_name = tkFileDialog.askdirectory(
                        parent=root, initialdir=USER_HOME, 
                        title='Please select an input directory')
    
        self.input_dir_display = '.../' + os.path.basename(self.input_dir_name)
        self.input_dir_chosen["text"] = self.input_dir_display
		
    
    def select_output_dir(self, event):
    
        self.output_dir_name = tkFileDialog.askdirectory(
                        parent=root, initialdir=USER_HOME, 
                        title='Please select an output directory')
        
        self.output_dir_display = '.../' + os.path.basename(self.output_dir_name)
        self.output_dir_chosen.config(text = self.output_dir_display)
		
	
    def submit_filename(self, event):
        
        self.output_file = self.output_file_entry.get() + ".csv"
        self.output_file_chosen["text"] = self.output_file
        
        
    def poll(self, progress_status):
        self.appParent.after(10, self.poll, progress_status)
        try:
            progress_status["text"] = self.queue.get(block=False)
        except Queue.Empty:
    		pass
        

    def run_program(self, event):
           
        if not self.input_dir_name:
            tkMessageBox.showinfo("Supply Information", "Choose Input Directory")
            
        if not self.output_dir_name:
            tkMessageBox.showinfo("Supply Information", "Choose Output Directory")
            
        if self.input_dir_name and self.output_dir_name:
            self.poll(self.progress_status)
            args = (self.input_dir_name, self.output_dir_name, self.output_file, self.queue)
            program_thread = Thread(target=main, args=args, kwargs=self.var_dict)
            program_thread.start()
                
            
class Catcher:
    
	def __init__(self, func, subst, widget):
		self.func = func
		self.subst = subst
		self.widget = widget

	def __call__(self, *args):
		try:
			if self.subst:
				args = apply(self.subst, args)
			return apply(self.func, args)
		except SystemExit, msg:
			raise SystemExit, msg
		except:
			ermessage = traceback.format_exc(1)
			ermessage = re.sub(r'.*(?=Error)', "", ermessage, flags=re.DOTALL)
			ermessage = "There was a problem processing your files:\n\n" + ermessage
			tkMessageBox.showerror("Error Message", ermessage)
            
            
if __name__ == '__main__':
    root = tk.Tk()
    root.wm_title("GAMET 1.0")
    root.configure(background=COLOR)
    root.geometry(GEOM_SIZE)
    tk.CallWrapper = Catcher
    app = App(root)
    root.mainloop()
